@@ -1,6 +1,7 @@
 print.vcov.gnm <- function(x, ...) {
-    if (!is.null(attr(x, "ofInterest")))
+    if (!is.null(attr(x, "ofInterest"))){
         print.default(x[attr(x, "ofInterest"), attr(x, "ofInterest")])
+    }
     else
         print.default(x)
 }
